<?php

namespace App\Api\Http\Controllers\Store;

use App\Core\Jobs\EditUserPodioJob;
use App\Core\Jobs\NewUserPodioJob;
use Carbon\Carbon;
use DB;
use App\Auth\Models\User;
use Illuminate\Http\Request;
use App\Admin\Role\Models\Role;
use App\Admin\Dashboard\Models\Car;
use Illuminate\Support\Facades\Log;
use App\Traits\Rest\ResponseHelpers;
use Illuminate\Pagination\Paginator;
use App\Transformers\Car\CarTransformer;
use App\Admin\Dashboard\Models\Store\Store;
use App\Transformers\Staff\StaffTransformer;
use App\Transformers\Store\InvitesTransformer;
use App\Http\Requests\Store\CreateStaffRequest;
use App\Http\Requests\Store\UpdateStaffRequest;
use App\Http\Requests\Store\InviteStaffRequest;
use App\Core\Jobs\Store\SendEmailInviteStaffJob;
use App\Core\Jobs\Store\SendEmailRegisterStaffJob;
use App\Core\Http\Controllers\CoreController as CoreController;

class StoreController extends CoreController
{
	use ResponseHelpers;

	protected $user;

	public function __construct()
    {
        $this->user = auth()->user();
    }

    /**
    * @SWG\Post(path="/store/{id}/staff",
    *    tags={"Store"}, summary="Register Staff", description="", operationId="createStaff", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( name="id", in="path", description="Min Lenght: 4 | Max Lenght: 255 ", required=true, type="string" ),
    *    @SWG\Parameter( name="name", in="formData", description="Min Lenght: 4 | Max Lenght: 255 ", required=true, type="string" ),
    *    @SWG\Parameter( name="phone", in="formData", description="", required=true, type="string" ),
    *    @SWG\Parameter( name="cell_phone", in="formData", description="", required=true, type="string" ),
    *    @SWG\Parameter( name="cpf", in="formData",  description="", required=true, type="string" ),
    *    @SWG\Parameter( name="email", in="formData",  description="", required=true, type="string" ),
    *    @SWG\Parameter( name="password", in="formData",  description="Min Lenght: 8", required=true, type="string" ),
    *    @SWG\Parameter( name="role", in="formData",  description="2=Diretor, 3=Gerente, 4=Vendedor", required=true, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function createStaff( CreateStaffRequest $request, $id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$user->hasRole( ['admin', 'manager', 'chief'] ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            $fields = $request->all();

            if ( $fields['role'] == 2 && !$user->hasRole( ['chief', 'admin'] ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( $fields['role'] == 3 && !$user->hasRole( ['chief', 'admin', 'manager'] ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            $role  = Role::find( $fields['role'] );
            $staff = new User();

            $staff->name      = $fields['name'];
            $staff->email     = $fields['email'];
            $staff->document  = $fields['cpf'];
            $staff->password  = $fields['password'];
            $staff->dealership = $store->trade_name;
            $staff->address = $store->street;
            $staff->number = $store->number;
            $staff->neighborhood = $store->neighborhood;
            $staff->zipcode = $store->zipcode;
            $staff->city = isset($store->city) ? $store->city->name : null;
            $staff->state = $store->city->state->initials;
            $staff->chat_token = uuidv4();
            $staff->phone = $request->get('phone', '');
            $staff->cell_phone = $fields['cell_phone'];
            $staff->role_id   = 2;
            // Preserva o tipo de usuário do cadastro principal (lojista/repassador/concessionária)
            $staff->user_type = $store->owner->user_type;
            $staff->save();

            $store->staff()->attach( $staff->id, ['role_id' => $fields['role'], 'status'=>1] );
            $staff->roles()->attach( $fields['role'] );

            $job = ( new SendEmailRegisterStaffJob( $staff, $store, $role ) )->onQueue( 'emails' );
            $this->dispatch( $job );

            $userJob = (new NewUserPodioJob($staff))->onQueue('push');
            $this->dispatch($userJob);

            return ( new StaffTransformer )->transform( $store->staff()->where( 'user_id', $staff->id )->first() );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/store/{id}/staff",
    *   tags={"Store"},
    *   summary="List Staff",
    *   description="",
    *   operationId="listStaff",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="query", name="per_page", description="Default: 20", required=false, type="integer" ),
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="id", description="", required=true, type="integer" ),
    *   @SWG\Parameter( in="query", name="current_page", description="Default: 1", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="order", description="ASC or DESC | Default: ASC", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="order_by", description="options: ID or created_at | Default:ID", required=false, type="string" ),
    *   @SWG\Response( response=200, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="status_code", type="integer", description="200" ),
    *           @SWG\Property( property="data", type="array",
    *               @SWG\Items( type="object",
    *                   @SWG\Property(property="totals", type="integer"),
    *                   @SWG\Property(property="per_page", type="integer"),
    *                   @SWG\Property(property="current_page", type="integer"),
    *                   @SWG\Property(property="items", type="array", @SWG\Items(
    *                       type="object", ref="#/definitions/Store"
    *                   ))
    *               )
    *           ),
    *       )
    *   ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   )
    * )
    */
    public function getAllStaff( Request $request, $id )
    {
        try {
            $per_page      = $request->input('per_page') ? $request->input('per_page') : 10;
            $current_page  = $request->input('current_page') ? $request->input('current_page') : 1;
            $order_by      = $request->input('order_by') ? $request->input('order_by') : 'created_at';
            $order         = $request->input('order') ? $request->input('order') : 'DESC';
            $user          = auth()->user();

            if ( !$store = Store::find( $id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }
            $particular = Store::getParticularStore();
            $corretor = Store::getCorretorStore();

            if ( ($store->id === $particular->id) || ($store->id === $corretor->id) ) {
                $staff = $store->staff()->where('store_user.user_id', $user->id)->first();
                return (new StaffTransformer)->transformCollectionByStore($staff, $store->id);
            }


            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( $user->hasRole( ['chief', 'admin'] ) ) {
                $roles = [2,3,4];
            } elseif ( $user->hasRole( ['manager'] ) ) {
                $roles = [3,4];
            } else {
                $roles = [4];
            }

            Paginator::currentPageResolver( function() use ( $current_page ) {
                return $current_page;
            });

            $staff = $store->staff()->whereIn( 'store_user.role_id', $roles )
                                    ->orderBy( $order_by, $order )
                                    ->paginate( $per_page );

            return ( new StaffTransformer )->transformCollectionByStore( $staff, $store->id );

        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/store/{store_id}/staff/{staff_id}",
    *    tags={"Store"}, summary="Update Staff", description="", operationId="updateStaff", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *    @SWG\Parameter( in="path", name="staff_id", description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="name", in="formData", description="Min Lenght: 4 | Max Lenght: 255 ", required=false, type="string" ),
    *    @SWG\Parameter( name="cell_phone", in="formData", description="", required=true, type="string" ),
    *    @SWG\Parameter( name="phone", in="formData", description="", required=true, type="string" ),
    *    @SWG\Parameter( name="cpf", in="formData",  description="", required=false, type="string" ),
    *    @SWG\Parameter( name="email", in="formData",  description="", required=false, type="string" ),
    *    @SWG\Parameter( name="password", in="formData",  description="Min Lenght: 8", required=false, type="string" ),
    *    @SWG\Parameter( name="role", in="formData",  description="2=Diretor, 3=Gerente, 4=Vendedor", required=false, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function updateStaff( UpdateStaffRequest $request, $store_id, $staff_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $staff_id )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$user->hasRole( ['admin', 'manager', 'chief'] ) && $user->id != $staff->id ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            $fields = $request->all();

            if ( isset( $fields['cpf'] ) ) {
                $fields['document'] = $fields['cpf'];
            }

            if ( isset( $fields['role'] ) && !$user->hasRole( ['admin', 'chief'] ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( isset( $fields['role'] ) ) {
                $store->staff()->detach( $staff->id );
                $store->staff()->attach( $staff->id, ['role_id' => $fields['role'], 'status'=>1] );
            }

            $staff->fill( $fields )->update();

            $editJob = (new EditUserPodioJob($staff, $this->user))->onQueue('push');
            $this->dispatch($editJob);

            return ( new StaffTransformer )->transform( $staff );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/store/{store_id}/staff/{staff_id}",
    *   tags={"Store"},
    *   summary="View a Staff",
    *   description="",
    *   operationId="viewStaff",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *   @SWG\Parameter( in="path", name="staff_id", description="", required=true, type="integer" ),
    *   @SWG\Response( response=200, description="", @SWG\Schema(ref="#/definitions/User") ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function viewStaff( $store_id, $staff_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $staff_id )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            } else {
                $belongs_store = true;
            }


            if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$user->hasRole( ['admin', 'chief', 'manager'] ) && $user->id != $staff->id ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            return ( new StaffTransformer )->transform( $staff );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Delete(path="/store/{store_id}/staff/{staff_id}",
    *   tags={"Store"},
    *   summary="Delete a Staff",
    *   description="",
    *   operationId="deleteStaff",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *   @SWG\Parameter( in="path", name="staff_id", description="", required=true, type="integer" ),
    *   @SWG\Response( response=204, description="Ok but not content" ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function removeStaff( $store_id, $staff_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $staff_id )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$user->hasRole( ['admin', 'chief'] ) && $user->id != $staff->id ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( $staff->anuncios->count() > 0 ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.dont_remove' ), 'errors'=>[]] , 400 );
            }

            $store->staff()->detach( $staff->id );

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.removed' )] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/store/{store_id}/staff/{staff_id}/invite",
    *    tags={"Store"}, summary="Invite Staff", description="", operationId="updateInvite", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *    @SWG\Parameter( in="path", name="staff_id", description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="role", in="formData",  description="2=Diretor, 3=Gerente, 4=Vendedor", required=false, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function inviteStaff( InviteStaffRequest $request, $store_id, $staff_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $staff_id ) {
                return $value->id == $staff_id;
            });

            if ( $belongs_store ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.store.belongs' ), 'errors'=>[]] , 400 );
            }

            $fields = $request->all();

            $store->staff()->attach( $staff_id, ['role_id' => $fields['role'], 'status'=>0] );

            $staff = $store->staff()->where( 'store_user.user_id', $staff_id )->first();
            $role  = Role::find( $fields['role'] );

            $job = ( new SendEmailInviteStaffJob( $staff, $store, $role ) )->onQueue( 'emails' );
            $this->dispatch( $job );

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.invite' )], 201 );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/store/{store_id}/accpet/invite",
    *    tags={"Store"}, summary="Accept Invite Staff", description="", operationId="accpetInvite", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function acceptInvite( $store_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $user->id )->where( 'store_user.status', 0 )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $user->roles()->attach($staff->pivot->role_id);
            $store->staff()->updateExistingPivot( $user->id, ['status'=>1] );

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.invite.accpet' )], 201 );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/store/{store_id}/reject/invite",
    *    tags={"Store"}, summary="Reject Invite Staff", description="", operationId="rejectInvite", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function rejectInvite( $store_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                return $value->id == $user->id;
            });

            if ( !$belongs_store ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $user->id )->where( 'store_user.status', 0 )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $store->staff()->detach( $staff->id );

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.removed' )] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/store/{store_id}/cancel/invite/staff/{staff_id}",
    *    tags={"Store"}, summary="Cancel Invite Staff", description="", operationId="rejectInvite", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *    @SWG\Parameter( in="path", name="staff_id", description="", required=true, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/User" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function cancelInvite( $store_id, $staff_id )
    {
        try {
            $user = auth()->user();
            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            if ( !$staff = $store->staff()->where( 'store_user.user_id', $staff_id )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $belongs_store = $store->staff->contains( function( $key, $value ) use( $staff ) {
                return $value->id == $staff->id;
            });

            if ( !$belongs_store ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            if ( !$store->staff()->where( 'store_user.user_id', $staff->id )->where( 'store_user.status', 0 )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $store->staff()->detach( $staff->id );

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.removed' )] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/store/staff/invites",
    *   tags={"Store"},
    *   summary="List All Invites",
    *   description="",
    *   operationId="listAllInvites",
    *   produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *   @SWG\Response( response=200, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="status_code", type="integer", description="200" ),
    *           @SWG\Property( property="data", type="array",
    *               @SWG\Items( type="object",
    *                   @SWG\Property(property="totals", type="integer"),
    *                   @SWG\Property(property="per_page", type="integer"),
    *                   @SWG\Property(property="current_page", type="integer"),
    *                   @SWG\Property(property="items", type="array", @SWG\Items(
    *                       type="object", ref="#/definitions/Store"
    *                   ))
    *               )
    *           ),
    *       )
    *   ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   )
    * )
    */
    public function getAllInvites()
    {
        try {
            $invites = auth()->user()->stores()->where( 'store_user.status', 0 )->get();

            return ( new InvitesTransformer )->transformCollection( $invites );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/store/{store_id}/cars",
    *   tags={"Store"},
    *   summary="List All Cars by store",
    *   description="",
    *   operationId="listAllCars",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *   @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
    *   @SWG\Parameter( in="query", name="current_page", description="Default: 1", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="order", description="ASC or DESC | Default: ASC", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="order_by", description="options: ID or created_at | Default:ID", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="roles", description="Enviar array com os ids das roles", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="user_id", description="", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="model", description="Pesquisar pelo nome do modelo do carro", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="status", description="1 = Em aberto, 2 = Negociando, 3 = Vendido, 4 = Em cadastro, 5 = Em analise, 6 = Suspenso, 7 = Bloqueado, 8 = Excluido, 9 = Turbo", required=false, type="integer" ),
    *   @SWG\Response( response=200, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="status_code", type="integer", description="200" ),
    *           @SWG\Property( property="data", type="array",
    *               @SWG\Items( type="object",
    *                   @SWG\Property(property="totals", type="integer"),
    *                   @SWG\Property(property="per_page", type="integer"),
    *                   @SWG\Property(property="current_page", type="integer"),
    *                   @SWG\Property(property="items", type="array", @SWG\Items(
    *                       type="object", ref="#/definitions/CarTransformer"
    *                   ))
    *               )
    *           ),
    *       )
    *   ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   )
    * )
    */
    public function getCarsByStore( Request $request, $store_id )
    {
        try {
            $user = auth()->user();

            if ( !$store = Store::find( $store_id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $staf_ids = null;

            $particular_store = Store::getParticularStore();
            $corretor_store = Store::getCorretorStore();

            if (($store->id === $particular_store->id) || ($store->id === $corretor_store->id) ) {
                $belongs_store = true;
                $request->merge(['user_id' => $user->id]);
                $staff_ids[] = $user->id;
            } else {
                $belongs_store = $store->staff->contains( function( $key, $value ) use( $user ) {
                    return $value->id == $user->id;
                });
            }

            if ( !$belongs_store ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }

            $staff_ids     = $staff_ids ?? $store->staff->pluck( 'id' )->toArray();
            $staff         = $request->input('user_id') ? [$request->input('user_id')] : null;
            $status        = $request->input('status') ? $request->input('status') : [1,2,3,4,5,6,7,9,10];
            $roles         = $request->input('roles') ? $request->input('roles') : null;
            $per_page      = $request->input('per_page') ? $request->input('per_page') : 10;
            $current_page  = $request->input('current_page') ? $request->input('current_page') : 1;
            $order_by      = $request->input('order_by') ? $request->input('order_by') : 'published_at';
            $order         = $request->input('order') ? $request->input('order') : 'DESC';
            $model         = $request->input('model') ? $request->input('model') : null;

            if ( is_null( $roles ) ) {
                // Caso seja administrador do sistema ou diretor da loja
                if ( $user->hasRole( ['admin'] ) || !is_null( $store->staff()->where( 'store_user.role_id', 2 )->where( 'store_user.user_id', $user->id )->first() ) ) {
                    $roles = [2,3,4];
                }
                // Caso seja gerente da loja
                elseif( $store->staff()->where( 'store_user.role_id', 3 )->where( 'store_user.user_id', $user->id )->first() ) {
                    $roles = [4];
                }
                // Caso seja vendedor da loja
                else{
                    $roles = [4];
                }
            } else {
                if ( $store->staff()->where( 'store_user.role_id', 4 )->where( 'store_user.user_id', $user->id )->first() ||
                     $store->staff()->where( 'store_user.role_id', 3 )->where( 'store_user.user_id', $user->id )->first() ) {
                    if ( count( $roles ) > 1 || !in_array( 4, $roles ) ) {
                        return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
                    }
                }

                if ( is_null( $staff ) ) {
                    $staff = $store->staff()->whereIn( 'store_user.role_id', $roles )->get()->pluck( 'id' )->toArray();
                }
            }

            if ( is_null( $staff ) ) {
                if ( $store->staff()->where( 'store_user.role_id', 2 )->where( 'store_user.user_id', $user->id )->first() ) {
                    $staff = $staff_ids;
                } elseif ( $store->staff()->where( 'store_user.role_id', 3 )->where( 'store_user.user_id', $user->id )->first() ) {
                    $staff = $store->staff()->where( 'store_user.role_id', 4 )->get()->pluck( 'id' )->toArray();
                    array_push( $staff, $user->id );
                } else {
                    $staff = [$user->id];
                }
            }

            Paginator::currentPageResolver( function() use ( $current_page ) {
                return $current_page;
            });


            $cars = Car::where( 'car.store_id', $store->id )
                                ->join( 'store_user', function ($join) use( $roles ) {
                                    $join->on( 'car.store_id', '=', 'store_user.store_id')->whereIn( 'store_user.role_id', $roles );
                                })

                                ->whereIn( 'car.user_id', $staff )
                                ->whereIn( 'car.car_status_id', $status )
                                ->whereLikeModel( $model )
                                ->select( 'car.*' )
                                ->orderBy( 'car.'.$order_by, $order )
                                ->orderByRaw('FIELD(car.car_status_id, 6, 7, 4, 1, 2, 3)') // Independente do status pré selecionado a ordem dos anúncios deve incluir essa sequência de status
                                ->groupBy( 'car.id' )
                                ->paginate( $per_page );

            return ( new CarTransformer )->transformCollection( $cars );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }
}