<?php

use App\Admin\Dashboard\Models\Car;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class ApiControllerSaveCarImageTest extends TestCase
{

    use DatabaseTransactions;

    public function testNoCarId()
    {
        $response = $this->call('POST', '/api/car-image/', [], [], [],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );
        $this->seeJson(['success' => false, 'msg' => 'Anúncio não informado'])->assertResponseStatus(400);
    }

    public function testNoFileSent()
    {
        $response = $this->call('POST', '/api/car-image/1702', [], [], [],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );
        $this->seeJson(['success' => false, 'msg' => 'Arquivo não enviado ou inválido'])->assertResponseStatus(400);

    }

    public function testFileUploadNotAllowed()
    {
        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/1702', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );

        $this->seeJson(['success' => false, 'msg' => 'Não autorizado'])->assertResponseStatus(403);

    }

    public function testFileUploadCarNotFound()
    {

        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/9999', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );

        $this->seeJson(['success' => false, 'msg' => 'Anúncio não localizado'])->assertResponseStatus(404);

    }

    public function testUploadNonEditableCar()
    {

        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/1489', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );

        $this->seeJson(['success' => false, 'msg' => 'Anúncio não pode ser editado'])->assertResponseStatus(401);
    }

    public function testFileUploadChangingStatus()
    {
        $car = Car::find(1706);
        $car->car_status_id = 6;
        $car->save();

        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/1706', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );

        $this->seeInDatabase('car', ['id' => $car->id, 'car_status_id' => 5]);
        $this->seeJson(['success' => true])->assertResponseOk();

    }

    public function testInvalidFileUpload()
    {

        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/gif', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/1706', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );


        $this->seeJson(['success' => false])->assertResponseStatus(400);

    }

    public function testFileUpload()
    {
        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/1706', [], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );

        $this->seeJson(['success' => true])->assertResponseOk();

    }

    public function testFirstPhotoUpload()
    {
        $car_id = 1706;
        $stub = __DIR__ .   '/../../../../stubs/test.jpg';
        $name = str_random(8).'.jpg';
        $path = sys_get_temp_dir().'/'.$name;

        copy($stub, $path);

        $file = new Illuminate\Http\UploadedFile($path, $name, 'image/jpeg', filesize($path), null, true);

        $response = $this->call('POST', '/api/car-image/' . $car_id, [
            'order' => 1
        ], [], [
            'filename' => $file
        ],
            [
                'HTTP_Authorization' => 'Basic ' . base64_encode('a@a.com' . ':' . '123'),
                'PHP_AUTH_USER' => 'a@a.com',
                'PHP_AUTH_PW' => '123'
            ]
        );
        $result = json_decode($response->getContent());

        // dd($data->data->id);

        $this->seeInDatabase('car_photo',
            [
                'id' => $result->data->id,
                'car_id' => $car_id,
                'filename' => $result->data->filename,
                'photo_order' => 1
            ]);
        $this->seeJson(['success' => true])->assertResponseOk();

    }



}
